 ###############################################################
 #                      MyPlainDigest                          #
 #                                                             #
 # MyPlainDigest provides an extension to the Mail::Internet   #
 # class that returns a plain text version of an email         #
 # message, suitable for use in plain text digests.            #
 #                                                             #
 # SYNOPSIS:                                                   #
 # (assuming an existing Mail::Internet object as $mail)       #
 #                                                             #
 # use MyPlainDigest;                                          #
 # $string = $mail->MyPlainDigest::plain_body_as_string(@opt); #
 #                                                             #
 # where @opt is an are options, currently:                    #
 # output_dir:  the directory under which temp files are stored#
 #                                                             #
 # WHAT DOES IT DO?                                            #
 # Most attachments are stripped out and replaced with a       #
 # note that they've been stripped. text/plain parts are       #
 # retained and encoding is 'levelled' to 8bit. A crude        #
 # attempt to convert single part text/html messages to plain  #
 # text is made. For text/plain parts that were not            #
 # originally in chasrset us-ascii or ISO-8859-1 all           #
 # characters above ascii 127 are replaced with '?' and a      #
 # warning added. Parts of type message/rfc822 are recursed    #
 # through in the same way, with brief headers included. Any   #
 # line consisting only of 30 hyphens has the first            #
 # character changed to space (see RFC 1153). Lines are        #
 # wrapped at 80 characters.                                   #
 #                                                             #
 # BUGS                                                        #
 # Probably dozens of them, and possibly dependant on your     #
 # versions of Perl and MIME-Tools (on which it is very        #
 # reliant).                                                   #
 # Seems to ignore any text after a UUencoded attachment.      #
 # Probably horrible if ISO-8859-1 or something close isn't    #
 # you're usual charset.                                       #
 #                                                             #
 # ACKNOWLEDGEMENTS                                            #
 # Written by an (c) Chris Hastie. _do_text_html largely       #
 # based on stripmime.pl by Alex Wetmore. Alex's original      #
 # boiler plate is at the end of this file.                    #
 # My bits you can do with as you wish.                        #
 #                                                             #
 #                                        Chris Hastie         #
 #                                                             #
 ###############################################################
 
 
 package MyPlainDigest;

 @ISA = qw(Mail::Internet);
 use Mail::Internet;
 use Mail::Address;
 use MIME::Parser;
 use Text::Wrap;
 use MIME::WordDecoder;
 use Language;
 
 sub plain_body_as_string {
 
  local $outstring = "";
  my ($mail, @paramlist) = @_;
  my %params = @paramlist;
  
  my $output_dir = $params{output_dir} || '.';
  

  my $parser = new MIME::Parser;
  $parser->extract_uuencode(1);  
  $parser->extract_nested_messages(1);
  $parser->output_dir($output_dir);

  # Convert Mail::Internet object to a MIME::Entity:
  my @lines = (@{$mail->header}, "\n", @{$mail->body});
  my $topent = $parser->parse_data(\@lines);
  
  #$topent->dump_skeleton; # for debugging only!
  
  _do_toplevel ($topent);

  # clean up after ourselves
  $topent->purge;
  
  $Text::Wrap::columns = 80;
  return wrap ('','',$outstring);
 }

 sub _do_toplevel {
 
  my $topent = shift;
  if ($topent->effective_type =~ /^text\/plain/i) {
    _do_text_plain($topent);
  }
  elsif ($topent->effective_type =~ /^text\/html/i) {
    _do_text_html($topent);
  }
  elsif ($topent->effective_type =~ /^multipart\/.*/i) {
    _do_multipart ($topent);
  }
  elsif ($topent->effective_type =~ /^message\/rfc822/i) {
    _do_message ($topent);
  } 
  elsif ($topent->effective_type =~ /^message\/delivery-status/i) {
    _do_dsn ($topent);
  }       
  else {
    _do_other ($topent);
  }
  return 1;
 }
 
 sub _do_multipart {

  my $topent = shift;

  # cycle through each part and process accordingly
  foreach $subent ($topent->parts) {    
     if ($subent->effective_type =~ /^text\/plain/i) {
       _do_text_plain($subent);
     }
     elsif ($subent->effective_type =~ /^multipart\/.*/i) {
       _do_multipart ($subent);
     } 
     elsif ($subent->effective_type =~ /^text\/html/i && $topent->effective_type =~ /^multipart\/alternative/i) {
       # assume there's a text/plain alternive, so don't warn
       # that the text/html part has been scrubbed
       next;
     }
     elsif ($subent->effective_type =~ /^message\/rfc822/i) {
       _do_message ($subent);
     } 
     elsif ($subent->effective_type =~ /^message\/delivery-status/i) {
       _do_dsn ($subent);
     }     
     else {
       _do_other ($subent);
     }
  }
  return 1;

 }
 
 sub _do_message {
  my $topent = shift;
  my $msgent = $topent->parts(0);
  my $wdecode = new MIME::WordDecoder::ISO_8859 (1);
  
  my $from = $msgent->head->get('From');
  my $subject = $msgent->head->get('Subject');
  my $date = $msgent->head->get('Date');
  my $to = $msgent->head->get('To');
  my $cc = $msgent->head->get('Cc');
  unless ($from = $wdecode->decode($from)) {
     $from = "???????";
  }
  unless ($to = $wdecode->decode($to)) {
     $to = "";
  }
  unless ($cc = $wdecode->decode($cc)) {
     $cc = "";
  } 
  unless ($subject = $wdecode->decode($subject)) {
     $subject = "";
  }
  chomp $from;
  chomp $to;
  chomp $cc;
  chomp $subject;
  chomp $date;
  
  my @fromline = Mail::Address->parse($from);
  my $name = $fromline[0]->name();
  $name = $fromline[0]->address() unless $name;

  $outstring .= sprintf (Msg (6, 301, "\n[Attached message follows]\n-----Original message-----\n" ),''); 
  $outstring .= "Date: $date\n" if $date;
  $outstring .= "From: $from\n" if $from;
  $outstring .= "To: $to\n" if $to;
  $outstring .= "Cc: $cc\n" if $cc;
  $outstring .= "Subject: $subject\n" if $subject;
  $outstring .= "\n";
  
  _do_toplevel ($msgent);
  
  $outstring .= sprintf (Msg (6, 302, "-----End of original message from %s-----\n\n"), $name);
  return 1;
 }

 sub _do_text_plain {
  my $entity = shift;    

  my $charset = $entity->head->mime_attr('content-type.charset'); 
  my $thispart;
  
  # this reads in the decoded body of the current entity  
  if ($io = $entity->open("r")) {
    while (defined($_ = $io->getline)) { 
      chomp $_;
      # if line is 30 hyphens, replace first character with space (RFC 1153)
      if ($_ eq "------------------------------") {
        s/^\-/ /;       
      }
      $thispart .= $_ . "\n";
    }
  }
  
  # scrub the 8bit characters (replace with '?') if the charset
  # isn't us-ascii or iso-8859-1. Add a warning.
  my %ok_charset = (
  	'us-ascii' => 1,
  	'iso-8859-1' => 1,
  	'iso_8859-1' => 1,
  	'latin1' => 1,
  	'latin-1' => 1,
  	'l1' => 1,
  	'windows-1252' => 1,
  	'iso-ir-100' => 1,
  	'ibm819' => 1,
  	'cp819' => 1,
  	'csisolatin1' => 1
  );
  if ($charset) {
    unless ($ok_charset{lc($charset)}) {
      $outstring .= sprintf (Msg (6, 303, "[** Warning: Message part originally used character set %s\n    Some characters may be lost or incorrect **]\n\n"), $charset);
      $thispart =~ tr/\x00-\x7F/\?/c;
    }
  }

  $outstring .= $thispart;
  return 1;
 }

 sub _do_other {
  # just add a note that attachment was stripped.
  my $entity = shift;
  $outstring .= sprintf (Msg (6, 304, "\n[An attachment of type %s was included here]\n"), $entity->mime_type);
  return 1;
 }
 
 sub _do_dsn {
   my $entity = shift;
   $outstring .= sprintf (Msg (6, 305, "\n-----Delivery Status Report-----\n"));
   _do_text_plain ($entity);
   $outstring .= sprintf (Msg (6, 306, "-----End of Delivery Status Report-----\n"));
 }

 sub _do_text_html {
  # Very crudely represent HTML part as plain text.
  # only done if top level CT is text/html (ie there
  # is no text/plain part). 
  #
  # Based on stripmime.pl by Alex Wetmore - see end of
  # file for original boilerplate

  my $entity = shift;

  # This is not at all complete.
  my %hHTMLCharConversion = (
    "nbsp"    => " ",
    "gt"    => ">",
    "lt"    => "<",
    "amp"   => "&",
    "and"   => "&&",
    "asymp"   => "=~",
    "brvbar"  => "|",
    "bull"    => "*",
    "cong"    => "=~",
    "copy"    => "(c)",
    "crarr"   => "<cr>",
    "equiv"   => "==",
    "ge"    => "<=",
    "lang"    => "<",
    "lsquo"   => "\'",
    "mdash"   => "--",
    "minus"   => "-",
    "reg"   => "(R)",
    "sim"   => "~",
    "thinsp"  => " ",
    "shy"   => "-",
    "trade"   => "(tm)",
    "times"   => "X",
    "pound"   => "",
  );

  my %hHTMLTagConversion = (
    "/div"    => "\n",
    "/p"    => "\n",
    "hr"    => "\n________________________________________________________________\n",
    "br"    => "\n",
    "li"    => " * ",
    "/tr"   => "\n",
  );

  my $fInTag;
  if ($io = $entity->open("r")) {
    while (defined($_ = $io->getline)) { 
       my $szThisLine = $_;
       chomp $szThisLine;
       my $szBeforeTag;
       my $szInTag;
       my $iStartTag;
       my $iEndTag;
       my $szAfterTag;
       my $szTagConversion = "";
       my $szTag;
       if ($fInTag) {
         if (($iEndTag = index($szThisLine, ">")) != -1) {
           $szThisLine = substr $szThisLine, $iEndTag+1;
           $fInTag = 0;
         } else {
           $szThisLine = "";
         }
       }
       while (($iStartTag = index($szThisLine, "<")) != -1) {
         $szBeforeTag = substr $szThisLine, 0, $iStartTag;
         $szInTag = substr $szThisLine, $iStartTag+1;
         if (($iEndTag = index($szInTag, ">")) != -1) {
           # we have the end of the tag on this line
           $szTag = substr $szInTag, 0, $iEndTag;
           $szAfterTag = substr $szInTag, $iEndTag+1;
         } else {
           # the tag extends to another line
           $szTag = $szInTag;
           $fInTag = 1;
           $szAfterTag = "";
         }
         $szTag = lc $szTag; 
         ($szTag, undef) = split (/\s/, $szTag, 2);
         if (exists $hHTMLTagConversion{$szTag}) {
           $szTagConversion = $hHTMLTagConversion{$szTag};
         }
         $szThisLine = "$szBeforeTag$szTagConversion$szAfterTag";
       }

       # now that we have removed all tags we go through and
       # convert special characters back to their ascii
       # equivelents
       while ($szThisLine =~ /^(.*)&(.*?);(.*)$/) {
         my $szBefore = $1;
         my $szAscii;
         my $szEntity = $2;
         my $szAfter = $3;
         if ($szEntity =~ /^#(\n\n\n)/) {
           if ($1 < 255) {
             # convert a numeric code
             $szAscii = chr $1;
           } else {
             $szAscii = "{$szEntity}";
           }
         } elsif (exists $hHTMLCharConversion{$szEntity}) {
           $szAscii = $hHTMLCharConversion{$szEntity};
         } else {
           # don't know how to convert
           $szAscii = "{$szEntity}";
         }
         $szThisLine = "$szBefore$szAscii$szAfter";       
       }
     
     # if line is 30 hyphens, replace first character with space (RFC 1153)
     if ($szThisLine =~ /^-{30}(\n|$)/) {
       $szThisLine =~ s/^\-/ /;       
     }         
     
     $outstring .= $szThisLine . "\n" unless ($szThisLine =~ /^\s*$/);
     }
   $io->close;
   }

 }
 1;

#####################################################################
#
# Code in _do_text_html is based largely on stripmime.pl by Alex 
# Wetmore. Below is the original boiler plate from that script.
#
########## BEGIN ALEX WETMORE'S COPYRIGHT NOTICE ####################
# This code is Copyright 2000-2001 Alex Wetmore. 
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met: 
#
# 1.  Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer. 
#
# 2.  Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following
# disclaimer in the documentation and/or other materials provided with
# the distribution. 
#
# 3.  All advertising materials mentioning features or use of this
# software must display the following acknowledgement:  This product
# includes software developed by Alex Wetmore. 
#
# 4.  The name of Alex Wetmore may not be used to endorse or promote
# products derived from this software without specific prior written
# permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS''
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
############# END ALEX WETMORE'S COPYRIGHT NOTICE ####################